/* Copyright (c) 2013, Mohamed Moubarak
 * Copyright (c) 2012, Marios Hadjieleftheriou
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.spatialindex;

import org.spatialindex.storagemanager.PropertySet;

public interface ISpatialIndex {
	public void flush() throws IllegalStateException;

	public void insertData(final byte[] data, final IShape shape, int id);

	public boolean deleteData(final IShape shape, int id);

	public void containmentQuery(final IShape query, final IVisitor v);

	public void intersectionQuery(final IShape query, final IVisitor v);

	public void pointLocationQuery(final IShape query, final IVisitor v);

	public void nearestNeighborQuery(int k, final IShape query, final IVisitor v, INearestNeighborComparator nnc);

	public void nearestNeighborQuery(int k, final IShape query, final IVisitor v);

	public void queryStrategy(final IQueryStrategy qs);

	public PropertySet getIndexProperties();

	public void addWriteNodeCommand(INodeCommand nc);

	public void addReadNodeCommand(INodeCommand nc);

	public void addDeleteNodeCommand(INodeCommand nc);

	public boolean isIndexValid();

	public IStatistics getStatistics();
} // ISpatialIndex

