/* Copyright (c) 2013, Mohamed Moubarak
 * Copyright (c) 2012, Marios Hadjieleftheriou
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.spatialindex;

import java.util.*;

public class RWLock {
	private int active_readers;
	private int waiting_readers;
	private int active_writers;

	private final LinkedList<Object> writer_locks = new LinkedList<Object>();

	public synchronized void read_lock() {
		if (active_writers == 0 && writer_locks.size() == 0)
			++active_readers;
		else {
			++waiting_readers;
			try {
				wait();
			}
			catch (InterruptedException e) {
			}
		}
	}

	public synchronized boolean read_lock_noblock() {
		if (active_writers == 0 && writer_locks.size() == 0) {
			++active_readers;
			return true;
		}
		return false;
	}

	public synchronized void read_unlock() {
		if (--active_readers == 0)
			notify_writers();
	}

	public void write_lock() {
		Object lock = new Object();
		synchronized (lock) {
			synchronized (this) {
				boolean okay_to_write = writer_locks.size() == 0 && active_readers == 0 && active_writers == 0;
				if (okay_to_write) {
					++active_writers;
					return; // the "return" jumps over the "wait" call
				}

				writer_locks.addLast(lock);
			}
			try {
				lock.wait();
			}
			catch (InterruptedException e) {
			}
		}
	}

	synchronized public boolean write_lock_noblock() {
		if (writer_locks.size() == 0 && active_readers == 0 && active_writers == 0) {
			++active_writers;
			return true;
		}
		return false;
	}

	public synchronized void write_unlock() {
		--active_writers;
		if (waiting_readers > 0) // priority to waiting readers
			notify_readers();
		else
			notify_writers();
	}

	private void notify_readers() // must be accessed from a
	{ // synchronized method
		active_readers += waiting_readers;
		waiting_readers = 0;
		notifyAll();
	}

	private void notify_writers() // must be accessed from a
	{ // synchronized method
		if (writer_locks.size() > 0) {
			Object oldest = writer_locks.removeFirst();
			++active_writers;
			synchronized (oldest) {
				oldest.notify();
			}
		}
	}
}
