/* Copyright (c) 2013, Mohamed Moubarak
 * Copyright (c) 2012, Marios Hadjieleftheriou
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.spatialindex.rtree;

import org.spatialindex.*;

import java.util.*;

public class Statistics implements IStatistics, Cloneable {
	protected long m_reads;
	protected long m_writes;
	protected long m_splits;
	protected long m_hits;
	protected long m_misses;
	protected long m_nodes;
	protected long m_adjustments;
	protected long m_queryResults;
	protected long m_data;
	protected int m_treeHeight;
	protected ArrayList<Integer> m_nodesInLevel = new ArrayList<Integer>();

	public Statistics() {
		reset();
	}

	public Statistics(Statistics s) {
		m_reads = s.m_reads;
		m_writes = s.m_writes;
		m_splits = s.m_splits;
		m_hits = s.m_hits;
		m_misses = s.m_misses;
		m_nodes = s.m_nodes;
		m_adjustments = s.m_adjustments;
		m_queryResults = s.m_queryResults;
		m_data = s.m_data;
		m_treeHeight = s.m_treeHeight;
		m_nodesInLevel = new ArrayList<Integer>(s.m_nodesInLevel);
	}

	public long getReads() {
		return m_reads;
	}

	public long getWrites() {
		return m_writes;
	}

	public long getNumberOfNodes() {
		return m_nodes;
	}

	public long getNumberOfData() {
		return m_data;
	}

	public long getSplits() {
		return m_splits;
	}

	public long getHits() {
		return m_hits;
	}

	public long getMisses() {
		return m_misses;
	}

	public long getAdjustments() {
		return m_adjustments;
	}

	public long getQueryResults() {
		return m_queryResults;
	}

	public int getTreeHeight() {
		return m_treeHeight;
	}

	public int getNumberOfNodesInLevel(int l) throws IndexOutOfBoundsException {
		return ((Integer) m_nodesInLevel.get(l)).intValue();
	}

	public void reset() {
		m_reads = 0;
		m_writes = 0;
		m_splits = 0;
		m_hits = 0;
		m_misses = 0;
		m_nodes = 0;
		m_adjustments = 0;
		m_queryResults = 0;
		m_data = 0;
		m_treeHeight = 0;
		m_nodesInLevel.clear();
	}

	public String toString() {
		String s = "Reads: " + m_reads + "\n" + "Writes: " + m_writes + "\n" + "Hits: " + m_hits + "\n" + "Misses: "
		                + m_misses + "\n" + "Tree height: " + m_treeHeight + "\n" + "Number of data: " + m_data + "\n"
		                + "Number of nodes: " + m_nodes + "\n";

		for (int cLevel = 0; cLevel < m_treeHeight; cLevel++) {
			s += "Level " + cLevel + " pages: " + ((Integer) m_nodesInLevel.get(cLevel)).intValue() + "\n";
		}

		s += "Splits: " + m_splits + "\n" + "Adjustments: " + m_adjustments + "\n" + "Query results: " + m_queryResults;

		return s;
	}

	public Object clone() {
		return new Statistics(this);
	}
}
