/* Copyright (c) 2013, Mohamed Moubarak
 * Copyright (c) 2012, Marios Hadjieleftheriou
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.spatialindex.storagemanager;

import java.util.*;

public abstract class Buffer implements IBuffer {
	int m_capacity = 10;
	boolean m_bWriteThrough = false;
	IStorageManager m_storageManager = null;
	HashMap<Integer, Buffer.Entry> m_buffer = new HashMap<Integer, Buffer.Entry>();
	long m_hits = 0;

	abstract void addEntry(int id, Buffer.Entry entry);

	abstract void removeEntry();

	public Buffer(IStorageManager sm, int capacity, boolean bWriteThrough) {
		m_storageManager = sm;
		m_capacity = capacity;
		m_bWriteThrough = bWriteThrough;
	}

	public byte[] loadByteArray(final int id) {
		byte[] ret = null;
		Entry e = (Entry) m_buffer.get(new Integer(id));

		if (e != null) {
			m_hits++;

			ret = new byte[e.m_data.length];
			System.arraycopy(e.m_data, 0, ret, 0, e.m_data.length);
		}
		else {
			ret = m_storageManager.loadByteArray(id);
			e = new Entry(ret);
			addEntry(id, e);
		}

		return ret;
	}

	public int storeByteArray(final int id, final byte[] data) {
		int ret = id;

		if (id == NewPage) {
			ret = m_storageManager.storeByteArray(id, data);
			Entry e = new Entry(data);
			addEntry(ret, e);
		}
		else {
			if (m_bWriteThrough) {
				m_storageManager.storeByteArray(id, data);
			}

			Entry e = (Entry) m_buffer.get(new Integer(id));
			if (e != null) {
				e.m_data = new byte[data.length];
				System.arraycopy(data, 0, e.m_data, 0, data.length);

				if (m_bWriteThrough == false) {
					e.m_bDirty = true;
					m_hits++;
				}
				else {
					e.m_bDirty = false;
				}
			}
			else {
				e = new Entry(data);
				if (m_bWriteThrough == false)
					e.m_bDirty = true;
				addEntry(id, e);
			}
		}

		return ret;
	}

	public void deleteByteArray(final int id) {
		Integer ID = new Integer(id);
		Entry e = (Entry) m_buffer.get(ID);
		if (e != null) {
			m_buffer.remove(ID);
		}

		m_storageManager.deleteByteArray(id);
	}

	public void flush() {
		Iterator<Map.Entry<Integer, Buffer.Entry>> it = m_buffer.entrySet().iterator();

		while (it.hasNext()) {
			Map.Entry<Integer, Buffer.Entry> me = it.next();
			Entry e = (Entry) me.getValue();
			int id = ((Integer) me.getKey()).intValue();
			if (e.m_bDirty)
				m_storageManager.storeByteArray(id, e.m_data);
		}

		m_storageManager.flush();
	}

	public void clear() {
		Iterator<Map.Entry<Integer, Buffer.Entry>> it = m_buffer.entrySet().iterator();

		while (it.hasNext()) {
			Map.Entry<Integer, Buffer.Entry> me = it.next();
			Entry e = (Entry) me.getValue();

			if (e.m_bDirty) {
				int id = ((Integer) me.getKey()).intValue();
				m_storageManager.storeByteArray(id, e.m_data);
			}
		}

		m_buffer.clear();
		m_hits = 0;
	}

	public long getHits() {
		return m_hits;
	}

	class Entry {
		byte[] m_data = null;
		boolean m_bDirty = false;

		Entry(final byte[] d) {
			m_data = new byte[d.length];
			System.arraycopy(d, 0, m_data, 0, d.length);
		}
	}; // Entry

} // Buffer
