/* Copyright (c) 2013, Mohamed Moubarak
 * Copyright (c) 2012, Marios Hadjieleftheriou
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.spatialindex.storagemanager;

import java.util.*;

public class MemoryStorageManager implements IStorageManager {
	private ArrayList<Entry> m_buffer = new ArrayList<Entry>();
	private Stack<Integer> m_emptyPages = new Stack<Integer>();

	public void flush() {
	}

	public byte[] loadByteArray(final int id) {
		Entry e = null;

		try {
			e = (Entry) m_buffer.get(id);
		}
		catch (IndexOutOfBoundsException ex) {
			throw new InvalidPageException(id);
		}

		byte[] ret = new byte[e.m_pData.length];
		System.arraycopy(e.m_pData, 0, ret, 0, e.m_pData.length);
		return ret;
	}

	public int storeByteArray(final int id, final byte[] data) {
		int ret = id;
		Entry e = new Entry(data);

		if (id == NewPage) {
			if (m_emptyPages.empty()) {
				m_buffer.add(e);
				ret = m_buffer.size() - 1;
			}
			else {
				ret = ((Integer) m_emptyPages.pop()).intValue();
				m_buffer.set(ret, e);
			}
		}
		else {
			if (id < 0 || id >= m_buffer.size())
				throw new InvalidPageException(id);
			m_buffer.set(id, e);
		}

		return ret;
	}

	public void deleteByteArray(final int id) {
		try {
			m_buffer.get(id);
		}
		catch (IndexOutOfBoundsException ex) {
			throw new InvalidPageException(id);
		}

		m_buffer.set(id, null);
		m_emptyPages.push(new Integer(id));
	}

	class Entry {
		byte[] m_pData;

		Entry(final byte[] d) {
			m_pData = new byte[d.length];
			System.arraycopy(d, 0, m_pData, 0, d.length);
		}
	} // Entry
}
