/* Copyright (c) 2013, Mohamed Moubarak
 * Copyright (c) 2012, Marios Hadjieleftheriou
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.spatialindex.storagemanager;

import java.util.*;

public class RandomEvictionsBuffer extends Buffer {
	Random m_random = new Random();

	public RandomEvictionsBuffer(IStorageManager sm, int capacity, boolean bWriteThrough) {
		super(sm, capacity, bWriteThrough);
	}

	void addEntry(int id, Entry e) {
		assert m_buffer.size() <= m_capacity;

		if (m_buffer.size() == m_capacity)
			removeEntry();
		m_buffer.put(new Integer(id), e);
	}

	void removeEntry() {
		if (m_buffer.size() == 0)
			return;

		int entry = m_random.nextInt(m_buffer.size());

		Iterator<Map.Entry<Integer, Buffer.Entry>> it = m_buffer.entrySet().iterator();
		for (int cIndex = 0; cIndex < entry - 1; cIndex++)
			it.next();

		Map.Entry<Integer, Entry> me = it.next();
		Entry e = (Entry) me.getValue();
		int id = ((Integer) me.getKey()).intValue();

		if (e.m_bDirty) {
			m_storageManager.storeByteArray(id, e.m_data);
		}

		m_buffer.remove(new Integer(id));
	}
} // RandomEvictionsBuffer
