/* Copyright (c) 2013, Mohamed Moubarak
 * Copyright (c) 2012, Marios Hadjieleftheriou
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.spatialindex;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.junit.Test;

import org.spatialindex.storagemanager.DiskStorageManager;
import org.spatialindex.storagemanager.IStorageManager;

public class TestBasic {

	@Test
	public void basic() throws SecurityException, NullPointerException, FileNotFoundException,
	                IllegalArgumentException, IOException {
		DiskStorageManager disk = new DiskStorageManager("foo", 1024);
		File a = new File("foo.dat");
		File b = new File("foo.idx");
		assertTrue(a.exists());
		assertTrue(b.exists());
		disk.flush();
		a.delete();
		b.delete();
	}

	@Test
	public void basic2() throws SecurityException, NullPointerException, FileNotFoundException,
	                IllegalArgumentException, IOException {
		boolean caught = false;
		try {
			new DiskStorageManager("bad/foo", 1024);
		}
		catch (IOException e) {
			caught = true;
		}
		assertTrue(caught);

	}

	@Test
	public void basic3() throws SecurityException, NullPointerException, FileNotFoundException,
	                IllegalArgumentException, IOException {
		DiskStorageManager disk = new DiskStorageManager("foo", 16);
		File a = new File("foo.dat");
		File b = new File("foo.idx");
		assertTrue(a.exists());
		assertTrue(b.exists());
		int first = disk.storeByteArray(IStorageManager.NewPage, "first one".getBytes());
		int id = 0;
		for (int i = 0; i < 100; i++) {
			id = disk.storeByteArray(IStorageManager.NewPage, ("for index " + i).getBytes());
		}
		assertEquals(disk.entries(), 101);
		byte[] entry = disk.loadByteArray(id);
		String result = new String(entry);
		assertEquals(result, "for index 99");
		result = new String(disk.loadByteArray(first));
		assertEquals(result, "first one");

		id = disk.storeByteArray(IStorageManager.NewPage, "1234567890abcdefghi".getBytes());

		for (int i = 0; i < 100; i++)
			disk.storeByteArray(IStorageManager.NewPage, ("filling" + i).getBytes());

		result = new String(disk.loadByteArray(id));
		assertEquals(result, "1234567890abcdefghi");
		disk.flush();

		a.delete();
		b.delete();
	}

}
